@@ -27,9 +27,14 @@
 #include "audits/memman.h"
 #include "sockets/socket.h"
 
+namespace cmdsocket {
+
+/** Command opcodes */
 enum t_cmd_code {
-	CMD_CALL,
-	CMD_CLI
+	CMD_CALL,	/**< Call */
+	CMD_CLI,	/**< Any CLI command */
+	CMD_SHOW,	/**< Show Twinkle */
+	CMD_HIDE	/**< Hide Twinkle */
 };
 
 string cmd_code2str(t_cmd_code opcode) {
@@ -38,6 +43,10 @@ string cmd_code2str(t_cmd_code opcode) {
 		return "CALL";
 	case CMD_CLI:
 		return "CLI";
+	case CMD_SHOW:
+		return "SHOW";
+	case CMD_HIDE:
+		return "HIDE";
 	default:
 		return "UNKNOWN";
 	}
@@ -52,19 +61,19 @@ void exec_cmd(t_socket_local &sock_client) {
 	try {
 		if (sock_client.read(&opcode, sizeof(opcode)) != sizeof(opcode)) {
 			log_file->write_report("Failed to read opcode from socket.",
-				"::exec_cmd", LOG_NORMAL, LOG_WARNING);
+				"cmdsocket::exec_cmd", LOG_NORMAL, LOG_WARNING);
 			return;
 		}
 		
 		if (sock_client.read(&immediate, sizeof(immediate)) != sizeof(immediate)) {
 			log_file->write_report("Failed to read immediate mode from socket.",
-				"::exec_cmd", LOG_NORMAL, LOG_WARNING);
+				"cmdsocket::exec_cmd", LOG_NORMAL, LOG_WARNING);
 			return;
 		}
 	
 		if (sock_client.read(&len, sizeof(len)) != sizeof(len)) {
 			log_file->write_report("Failed to read length from socket.",
-				"::exec_cmd", LOG_NORMAL, LOG_WARNING);
+				"cmdsocket::exec_cmd", LOG_NORMAL, LOG_WARNING);
 			return;
 		}
 		
@@ -72,11 +81,11 @@ void exec_cmd(t_socket_local &sock_client) {
 		
 		if (sock_client.read(args, len) != len) {
 			log_file->write_report("Failed to read arguments from socket.",
-				"::exec_cmd", LOG_NORMAL, LOG_WARNING);
+				"cmdsocket::exec_cmd", LOG_NORMAL, LOG_WARNING);
 			return;
 		}
 		
-		log_file->write_header("::exec_cmd", LOG_NORMAL, LOG_DEBUG);
+		log_file->write_header("cmdsocket::exec_cmd", LOG_NORMAL, LOG_DEBUG);
 		log_file->write_raw("External command received:\n");
 		log_file->write_raw("Opcode: ");
 		log_file->write_raw(cmd_code2str(opcode));
@@ -94,8 +103,24 @@ void exec_cmd(t_socket_local &sock_client) {
 		case CMD_CLI:
 			ui->cmd_cli(args, immediate);
 			break;
+		case CMD_SHOW:
+			ui->cmd_show();
+			break;
+		case CMD_HIDE:
+			ui->cmd_hide();
+			break;
 		default:
 			// Discard unknown commands
+			log_file->write_header("cmdsocket::exec_cmd", LOG_NORMAL, LOG_WARNING);
+			log_file->write_raw("Unknown external command received:\n");
+			log_file->write_raw("Opcode: ");
+			log_file->write_raw(cmd_code2str(opcode));
+			log_file->write_raw("\nImmediate: ");
+			log_file->write_raw(bool2yesno(immediate));
+			log_file->write_raw("\nArguments: ");
+			log_file->write_raw(args);
+			log_file->write_endl();
+			log_file->write_footer();
 			break;
 		}
 	}
@@ -103,7 +128,7 @@ void exec_cmd(t_socket_local &sock_client) {
 		log_msg = "Failed to read from socket.\n";
 		log_msg += strerror(e);
 		log_msg += "\n";
-		log_file->write_report(log_msg, "::exec_cmd", LOG_NORMAL, LOG_WARNING);
+		log_file->write_report(log_msg, "cmdsocket::exec_cmd", LOG_NORMAL, LOG_WARNING);
 	}
 }
 
@@ -121,14 +146,14 @@ void *listen_cmd(void *arg) {
 			log_msg = "Accept failed on socket.\n";
 			log_msg += strerror(e);
 			log_msg += "\n";
-			log_file->write_report(log_msg, "::listen_cmd", LOG_NORMAL, 
+			log_file->write_report(log_msg, "cmdsocket::listen_cmd", LOG_NORMAL, 
 				LOG_WARNING);
 			return NULL;
 		}
 	}
 }
 
-void cmd_call(const string &destination, bool immediate) {
+void write_cmd_to_socket(t_cmd_code opcode, bool immediate, const string &args) {
 	string name = sys_config->get_dir_user();
 	name += '/';
 	name += CMD_SOCKNAME;
@@ -136,12 +161,11 @@ void cmd_call(const string &destination, bool immediate) {
 	try {
 		t_socket_local sock_cmd;
 		sock_cmd.connect(name);
-		t_cmd_code opcode = CMD_CALL;
 		sock_cmd.write(&opcode, sizeof(opcode));
 		sock_cmd.write(&immediate, sizeof(immediate));
-		int len = destination.size() + 1;
+		int len = args.size() + 1;
 		sock_cmd.write(&len, sizeof(len));
-		char *buf = strdup(destination.c_str());
+		char *buf = strdup(args.c_str());
 		MEMMAN_NEW(buf);
 		sock_cmd.write(buf, len);
 		MEMMAN_DELETE(buf);
@@ -152,36 +176,25 @@ void cmd_call(const string &destination, bool immediate) {
 		// notices another Twinkle is already running. In that
 		// case this process does not have a log file. So write
 		// errors to stderr
-		cerr << "Failed to send call command to " << name << endl;
+		cerr << "Failed to send " << cmd_code2str(opcode) << " command to " << name << endl;
 		cerr << strerror(e) << endl;
 	}
 }
 
+void cmd_call(const string &destination, bool immediate) {
+	write_cmd_to_socket(CMD_CALL, immediate, destination);
+}
+
 void cmd_cli(const string &cli_command, bool immediate) {
-	string name = sys_config->get_dir_user();
-	name += '/';
-	name += CMD_SOCKNAME;
+	write_cmd_to_socket(CMD_CLI, immediate, cli_command);
+}
 
-	try {
-		t_socket_local sock_cmd;
-		sock_cmd.connect(name);
-		t_cmd_code opcode = CMD_CLI;
-		sock_cmd.write(&opcode, sizeof(opcode));
-		sock_cmd.write(&immediate, sizeof(immediate));
-		int len = cli_command.size() + 1;
-		sock_cmd.write(&len, sizeof(len));
-		char *buf = strdup(cli_command.c_str());
-		MEMMAN_NEW(buf);
-		sock_cmd.write(buf, len);
-		MEMMAN_DELETE(buf);
-		free(buf);
-	}
-	catch (int e) {
-		// This function will be called from Twinkle when it
-		// notices another Twinkle is already running. In that
-		// case this process does not have a log file. So write
-		// errors to stderr
-		cerr << "Failed to send CLI command to " << name << endl;
-		cerr << strerror(e) << endl;
-	}
+void cmd_show(void) {
+	write_cmd_to_socket(CMD_SHOW, true, "");
+}
+
+void cmd_hide(void) {
+	write_cmd_to_socket(CMD_HIDE, true, "");
+}
+
 }
