@@ -137,6 +137,7 @@ void MphoneForm::init()
 		serviceDnd->addTo(menu);
 		serviceRedirection->addTo(menu);
 		serviceAutoAnswer->addTo(menu);
+		servicesVoice_mailAction->addTo(menu);
 		
 		menu->insertSeparator();
 		
@@ -2574,6 +2575,11 @@ void MphoneForm::popupMenuVoiceMail(const QPoint &pos)
 	}
 }
 
+void MphoneForm::popupMenuVoiceMail(void)
+{
+	popupMenuVoiceMail(QCursor::pos());
+}
+
 void MphoneForm::showDisplay(bool on)
 {
 	if (on) {
