@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,29 +20,13 @@
 #include "definitions.h"
 #include "parse_ctrl.h"
 
-t_hdr_content_type::t_hdr_content_type() : t_header() {};
+t_hdr_content_type::t_hdr_content_type() : t_header("Content-Type", "c") {};
 
 void t_hdr_content_type::set_media(const t_media &m) {
 	populated = true;
 	media = m;
 }
 
-string t_hdr_content_type::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	if (t_parser::compact_headers) {
-		s = "c: ";
-	} else {
-		s = "Content-Type: ";
-	}
-
-	s += encode_value();
-	s += CRLF;
-	return s;
-}
-
 string t_hdr_content_type::encode_value(void) const {
 	string s;
 
@@ -51,12 +35,3 @@ string t_hdr_content_type::encode_value(void) const {
 	s = media.encode();
 	return s;
 }
-
-string t_hdr_content_type::encode_env(void) const {
-	string s;
-	
-	s = "SIP_CONTENT_TYPE=";
-	s += encode_value();
-	
-	return s;
-}
