@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,9 +32,7 @@ public:
 	t_hdr_date();
 	void set_date_gm(struct tm *tm); // set date, tm is GMT
 	void set_now(void); // Set date/time to current date/time
-	string encode(void) const;
 	string encode_value(void) const;
-	string encode_env(void) const;
 };
 
 using namespace std;
