@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #include "hdr_reply_to.h"
 #include "definitions.h"
 
-t_hdr_reply_to::t_hdr_reply_to() : t_header() {}
+t_hdr_reply_to::t_hdr_reply_to() : t_header("Reply-To") {}
 
 void t_hdr_reply_to::set_display(const string &d) {
 	populated = true;
@@ -46,18 +46,6 @@ void t_hdr_reply_to::add_param(const t_parameter &p) {
 	params.push_back(p);
 }
 
-string t_hdr_reply_to::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	s = "Reply-To: ";
-	s += encode_value();
-	s += CRLF;
-
-	return s;
-}
-
 string t_hdr_reply_to::encode_value(void) const {
 	string s;
 
@@ -78,12 +66,3 @@ string t_hdr_reply_to::encode_value(void) const {
 
 	return s;
 }
-
-string t_hdr_reply_to::encode_env(void) const {
-	string s;
-	
-	s = "SIP_REPLY_TO=";
-	s += encode_value();
-	
-	return s;
-}
